import json

from wayscript import utils
from wayscript.errors import MissingCredentialsError


PSYCOPG2_CONNECTION_KWARG_MAP = {
        "dbname": "database_name",
        "user": "database_user",
        "password": "database_password",
        "host": "database_host",
        "port": "database_port",
}


MSSQL_CONNECTION_KWARG_MAP = {
    "server": "database_host",
    "user": "database_user",
    "password": "database_password",
    "database": "database_name",
    "port": "database_port",

}


MYSQL_CONNECTION_KWARG_MAP = {
    "host": "database_host",
    "user": "database_user",
    "password": "database_password",
    "database": "database_name",
    "port": "database_port",
}


def get_connection_kwargs(_id: str, credentials_mapping: dict) -> dict:
    """
    Return connection kwargs
    
    If you want to instantiate your own client, use this method.
    """
    wayscript_client = utils.WayScriptClient()
    response = wayscript_client.get_workspace_integration_detail(_id)
    response.raise_for_status()
    workspace_integration_data = response.json()
    credentials_str = workspace_integration_data.get("credentials")
    credentials = {}

    try:
        credentials = json.loads(credentials_str)
    except json.decoder.JSONDecodeError:
        credentials = {}

    kwargs = {k: credentials.get(v) for k, v in credentials_mapping.items()}

    if not credentials:
        raise MissingCredentialsError(f"Missing credentials for workspace_integration={_id}")

    return kwargs

def get_psycopg2_connection_kwargs(_id: str) -> dict:
    """Get connection kwargs for psycopg2"""
    return get_connection_kwargs(_id, credentials_mapping=PSYCOPG2_CONNECTION_KWARG_MAP)

def get_mssql_connection_kwargs(_id: str) -> dict:
    """Get connection kwargs for SQL Server connection via mssql driver"""
    return get_connection_kwargs(_id, credentials_mapping=MSSQL_CONNECTION_KWARG_MAP)

def get_mysql_connection_kwargs(_id: str) -> dict:
    """Return connection kwargs for MySQL connection via mysql driver"""
    return get_connection_kwargs(_id, credentials_mapping=MYSQL_CONNECTION_KWARG_MAP)



from functools import lru_cache

from . import utils


@lru_cache()
def _get_process_detail_expanded_data() -> dict:
    """Wrapper to cache data from process_detail_expanded call"""
    process_id = utils.get_process_id()
    client = utils.WayScriptClient()
    response = client.get_process_detail_expanded(process_id)
    response.raise_for_status()
    return response.json()


def get_process():
    """
    Return metadata about the current execution process.
    """
    data = _get_process_detail_expanded_data()["process"]
    return data


def get_event():
    """Returns the event object of the execution context"""
    data = _get_process_detail_expanded_data()["event"]
    return data


def get_lair_trigger():
    """Returns information about the trigger that launched the event"""
    data = _get_process_detail_expanded_data()["lair_trigger"]
    return data


def get_lair():
    """Returns lair metadata"""
    process = get_process()
    lair_id = process["lair_id"]
    client = utils.WayScriptClient()
    response = client.get_lair_detail(lair_id)
    response.raise_for_status()
    data = response.json()
    return data


def get_workspace():
    """Return workspace metadata"""
    lair_data = get_lair()
    workspace_id = lair_data["workspace_id"]
    client = utils.WayScriptClient()
    response = client.get_workspace_detail(workspace_id)
    response.raise_for_status()
    data = response.json()
    return data


def get_user_by_application_key(application_key: str) -> dict:
    """Return a user object for the given application key"""
    lair_data = get_lair()
    workspace_id = lair_data["workspace_id"]
    client = utils.WayScriptClient()
    response = client.get_user_detail_by_application_key(application_key, workspace_id)
    response.raise_for_status()
    data = response.json()
    return data



"""
Module for settings and configs
"""
import os

AUTH_ROUTE = "auth"
PROCESSES_ROUTE = "processes"
LAIRS_ROUTE = "lairs"
WEBHOOKS = "webhooks"
WORKSPACES_ROUTE = "workspaces"
WORKSPACE_INTEGRATIONS_ROUTE = "workspace-integrations"



ROUTES = {
    "auth": {"refresh": f"{AUTH_ROUTE}/refresh"},
    "lairs": {"detail": f"{LAIRS_ROUTE}/$id"},
    "processes": { "detail_expanded": f"{PROCESSES_ROUTE}/$id/detail"},
    "webhooks": {"http_trigger_response": f"{WEBHOOKS}/http-trigger/response/$id"},
    "workspaces": {
        "detail": f"{WORKSPACES_ROUTE}/$id",
        "user_application_key_detail": f"{WORKSPACES_ROUTE}/$id/users/self",
        },
    "workspace-integrations": {"detail": f"{WORKSPACE_INTEGRATIONS_ROUTE}/$id"},
}

# origin is scheme + domain + port. explanation: https://stackoverflow.com/a/37366696/4293004
WAYSCRIPT_ORIGIN = os.environ.get("WAYSCRIPT_ORIGIN", "https://api.wayscript.com")



import functools
import os
import string

import requests

from . import settings


def get_process_execution_user_token():
    """Return the auth token of the user this process is executing on behalf of"""
    token = os.environ.get("WAYSCRIPT_EXECUTION_USER_TOKEN")
    return token

def set_process_execution_user_token(value: str):
    os.environ["WAYSCRIPT_EXECUTION_USER_TOKEN"] = value
    return value

def get_process_id():
    """Return uuid of current container execution"""
    process_id = os.environ["WS_PROCESS_ID"]
    return process_id


def get_refresh_token():
    refresh_token = os.environ["WAYSCRIPT_EXECUTION_USER_REFRESH_TOKEN"]
    return refresh_token

def get_application_key():
    application_key = os.environ["WAYSCRIPT_EXECUTION_USER_APPLICATION_KEY"]
    return application_key


def retry_on_401_wrapper(f):

    @functools.wraps(f)
    def _retry_on_401(client, *args, **kwargs):

        response = f(client, *args, **kwargs)
        # refresh credentials and retry on 401
        if response.status_code == 401:
            client._refresh_access_token()
            response = f(client, *args, **kwargs)
        return response

    return _retry_on_401


class WayScriptClient:

    def __init__(self, *args, **kwargs):
        """Init a wayscript client"""
        self.session = requests.Session()
        access_token =  get_process_execution_user_token()
        self.session.headers["authorization"] = f"Bearer {access_token}"
        self.session.headers["content-type"] = "application/json"
    
    def _get_url(self, subpath: str, route: str, template_args: dict=None):
        """Generate an url"""
        subpath_template_str = settings.ROUTES[subpath][route]
        subpath_template = string.Template(subpath_template_str)
        full_subpath = subpath_template.substitute(**template_args) if template_args else subpath_template_str

        url = f"{settings.WAYSCRIPT_ORIGIN}/{full_subpath}"
        return url

    def _refresh_access_token(self):
        """Refresh access token and update environment"""
        url = self._get_url(subpath="auth", route="refresh")
        refresh_token = get_refresh_token()
        payload = {"refresh_token": refresh_token}
        response = self.session.post(url, json=payload)
        response.raise_for_status()
        access_token = response.json()["access_token"]
        set_process_execution_user_token(access_token)
        self.session.headers["authorization"] = f"Bearer {access_token}"

    @retry_on_401_wrapper
    def get_process_detail_expanded(self, _id: str):
        """Request process expanded detail endpoint"""
        url = self._get_url(subpath="processes", route="detail_expanded", template_args={"id": _id})
        response = self.session.get(url)
        return response

    @retry_on_401_wrapper
    def get_workspace_integration_detail(self, _id: str):
        """Request a workspace-integrations detail"""
        url = self._get_url(subpath="workspace-integrations", route="detail", template_args={"id": _id})
        response = self.session.get(url)
        return response

    @retry_on_401_wrapper
    def get_lair_detail(self, _id: str):
        """Request lair detail"""
        url = self._get_url(subpath="lairs", route="detail", template_args={"id": _id})
        response = self.session.get(url)
        return response

    @retry_on_401_wrapper
    def get_workspace_detail(self, _id: str):
        """Request workspace detail"""
        url = self._get_url(subpath="workspaces", route="detail", template_args={"id": _id})
        response = self.session.get(url)
        return response

    @retry_on_401_wrapper
    def post_webhook_http_trigger_response(self, _id: str, payload: dict=None):
        """
        Post an http trigger response

        _id: process id launched by http trigger
        payload: a payload describing how to respond to the http trigger's request
        """
        url = self._get_url(subpath="webhooks", route="http_trigger_response", template_args={"id": _id})
        response = self.session.post(url, json=payload)
        return response

    def get_user_detail_by_application_key(self, application_key: str, workspace_id: str):
        """
        Request user detail using its application key
        """
        url = self._get_url(subpath="workspaces", route="user_application_key_detail", template_args={"id": workspace_id})
        response = self.session.get(url, headers={"authorization": f"Bearer {application_key}"})
        return response



#!/usr/bin/env python

# Copyright (c) 2019 WayScript, Inc. All rights reserved.
# Licensed under the MIT License.

import setuptools

with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()


setuptools.setup(
    name="wayscript",
    version='0.2.2',
    author="Team WayScript",
    author_email="founders@wayscript.com",
    description="WayScript gives you flexible building blocks to seamlessly integrate, automate and host tools in the cloud.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['slack-sdk', 'requests>=2.22.0'],
    url="https://github.com/wayscript/wayscript-python",
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where="src"),
    license='MIT',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications",
        "Topic :: Database",
        "Topic :: Office/Business",
        "Topic :: Utilities",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
    ],
    keywords=[ 'wayscript', 'productivity', 'software', 'superpowers', 'scripts', 'cloud', 'tools', 'backend',
               'visual', 'low-code', 'modules', 'trigger', 'integration', 'dev', 'http', 'webhook' ],
    zip_safe=False,
)



from wayscript import utils


def send_response(data: dict=None, headers: dict=None, status_code: int=None):
    """Send response to http trigger endpoint"""
    assert data, "data kwarg is required"
    assert headers, "headers kwarg is required"
    assert status_code, "status_code kwarg is required"
    wayscript_client = utils.WayScriptClient()
    process_id = utils.get_process_id()
    payload = {
        "data": data,
        "headers": headers,
        "status_code": status_code,
    }
    response = wayscript_client.post_webhook_http_trigger_response(process_id, payload)
    response.raise_for_status()

    return response



import json

from slack_sdk import WebClient

from wayscript import utils
from wayscript.errors import MissingCredentialsError


def get_client_for_workspace_integration(_id: str) -> WebClient:
    """Get a slack client with credentials from a workspace integration"""
    wayscript_client = utils.WayScriptClient()
    response = wayscript_client.get_workspace_integration_detail(_id)
    response.raise_for_status()
    workspace_integration_data = response.json()

    access_token = None
    credentials_str = workspace_integration_data["credentials"]

    try:
        credentials = json.loads(credentials_str)
        access_token = credentials.get("access_token")
    except json.decoder.JSONDecodeError:
        access_token = None
    
    if not access_token:
        raise MissingCredentialsError(f"No credentials found for {_id}")
    else:
        return WebClient(access_token)
